# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pkg/kast/interface/qt5/view/MediaControlView.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MediaControlView(object):
    def setupUi(self, MediaControlView):
        MediaControlView.setObjectName("MediaControlView")
        MediaControlView.resize(849, 112)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(MediaControlView)
        self.verticalLayout_2.setContentsMargins(0, 6, 0, 6)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self._layout = QtWidgets.QVBoxLayout()
        self._layout.setObjectName("_layout")
        self._layoutMediaControl = QtWidgets.QGridLayout()
        self._layoutMediaControl.setObjectName("_layoutMediaControl")
        self.buttonPlayPause = QtWidgets.QPushButton(MediaControlView)
        self.buttonPlayPause.setText("")
        self.buttonPlayPause.setObjectName("buttonPlayPause")
        self._layoutMediaControl.addWidget(self.buttonPlayPause, 1, 3, 1, 1)
        self.buttonSeekFront = QtWidgets.QPushButton(MediaControlView)
        self.buttonSeekFront.setText("")
        self.buttonSeekFront.setObjectName("buttonSeekFront")
        self._layoutMediaControl.addWidget(self.buttonSeekFront, 1, 5, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self._layoutMediaControl.addItem(spacerItem, 1, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self._layoutMediaControl.addItem(spacerItem1, 1, 6, 1, 1)
        self.buttonSeekBack = QtWidgets.QPushButton(MediaControlView)
        self.buttonSeekBack.setText("")
        self.buttonSeekBack.setObjectName("buttonSeekBack")
        self._layoutMediaControl.addWidget(self.buttonSeekBack, 1, 2, 1, 1)
        self._layoutLeftSideControls = QtWidgets.QGridLayout()
        self._layoutLeftSideControls.setObjectName("_layoutLeftSideControls")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self._layoutLeftSideControls.addItem(spacerItem2, 2, 1, 1, 1)
        self.buttonDisconnect = QtWidgets.QPushButton(MediaControlView)
        self.buttonDisconnect.setObjectName("buttonDisconnect")
        self._layoutLeftSideControls.addWidget(self.buttonDisconnect, 0, 0, 1, 1)
        self.labelTime = QtWidgets.QLabel(MediaControlView)
        self.labelTime.setStyleSheet("font: bold")
        self.labelTime.setObjectName("labelTime")
        self._layoutLeftSideControls.addWidget(self.labelTime, 2, 0, 1, 1)
        self._layoutMediaControl.addLayout(self._layoutLeftSideControls, 1, 0, 1, 1)
        self._layoutRightSideControls = QtWidgets.QGridLayout()
        self._layoutRightSideControls.setHorizontalSpacing(10)
        self._layoutRightSideControls.setVerticalSpacing(0)
        self._layoutRightSideControls.setObjectName("_layoutRightSideControls")
        self.sliderVolumeLocal = QtWidgets.QSlider(MediaControlView)
        self.sliderVolumeLocal.setMaximum(100)
        self.sliderVolumeLocal.setProperty("value", 100)
        self.sliderVolumeLocal.setOrientation(QtCore.Qt.Horizontal)
        self.sliderVolumeLocal.setInvertedAppearance(False)
        self.sliderVolumeLocal.setInvertedControls(False)
        self.sliderVolumeLocal.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.sliderVolumeLocal.setTickInterval(10)
        self.sliderVolumeLocal.setObjectName("sliderVolumeLocal")
        self._layoutRightSideControls.addWidget(self.sliderVolumeLocal, 1, 2, 1, 1)
        self.sliderVolumeRemote = QtWidgets.QSlider(MediaControlView)
        self.sliderVolumeRemote.setMaximum(100)
        self.sliderVolumeRemote.setProperty("value", 100)
        self.sliderVolumeRemote.setOrientation(QtCore.Qt.Horizontal)
        self.sliderVolumeRemote.setInvertedAppearance(False)
        self.sliderVolumeRemote.setInvertedControls(False)
        self.sliderVolumeRemote.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.sliderVolumeRemote.setTickInterval(10)
        self.sliderVolumeRemote.setObjectName("sliderVolumeRemote")
        self._layoutRightSideControls.addWidget(self.sliderVolumeRemote, 2, 2, 1, 1)
        self.buttonMuteLocal = QtWidgets.QPushButton(MediaControlView)
        self.buttonMuteLocal.setText("")
        self.buttonMuteLocal.setObjectName("buttonMuteLocal")
        self._layoutRightSideControls.addWidget(self.buttonMuteLocal, 1, 3, 1, 1)
        self.labelVolumeRemote = QtWidgets.QLabel(MediaControlView)
        self.labelVolumeRemote.setObjectName("labelVolumeRemote")
        self._layoutRightSideControls.addWidget(self.labelVolumeRemote, 2, 1, 1, 1)
        self.labelVolumeLocal = QtWidgets.QLabel(MediaControlView)
        self.labelVolumeLocal.setObjectName("labelVolumeLocal")
        self._layoutRightSideControls.addWidget(self.labelVolumeLocal, 1, 1, 1, 1)
        self.buttonMuteRemote = QtWidgets.QPushButton(MediaControlView)
        self.buttonMuteRemote.setText("")
        self.buttonMuteRemote.setObjectName("buttonMuteRemote")
        self._layoutRightSideControls.addWidget(self.buttonMuteRemote, 2, 3, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self._layoutRightSideControls.addItem(spacerItem3, 2, 0, 1, 1)
        self._layoutMediaControl.addLayout(self._layoutRightSideControls, 1, 7, 1, 1)
        self.buttonStop = QtWidgets.QPushButton(MediaControlView)
        self.buttonStop.setText("")
        self.buttonStop.setObjectName("buttonStop")
        self._layoutMediaControl.addWidget(self.buttonStop, 1, 4, 1, 1)
        self._layoutMediaControl.setColumnStretch(0, 1)
        self._layoutMediaControl.setColumnStretch(1, 1)
        self._layoutMediaControl.setColumnStretch(6, 1)
        self._layoutMediaControl.setColumnStretch(7, 1)
        self._layout.addLayout(self._layoutMediaControl)
        self.sliderSeek = QtWidgets.QSlider(MediaControlView)
        self.sliderSeek.setOrientation(QtCore.Qt.Horizontal)
        self.sliderSeek.setObjectName("sliderSeek")
        self._layout.addWidget(self.sliderSeek)
        self.verticalLayout_2.addLayout(self._layout)

        self.retranslateUi(MediaControlView)
        QtCore.QMetaObject.connectSlotsByName(MediaControlView)

    def retranslateUi(self, MediaControlView):
        _translate = QtCore.QCoreApplication.translate
        MediaControlView.setWindowTitle(_translate("MediaControlView", "MediaControlView"))
        self.buttonPlayPause.setToolTip(_translate("MediaControlView", "Play/Pause"))
        self.buttonSeekFront.setToolTip(_translate("MediaControlView", "Seek front"))
        self.buttonSeekBack.setToolTip(_translate("MediaControlView", "Seek back"))
        self.buttonDisconnect.setText(_translate("MediaControlView", "Disconnect"))
        self.labelTime.setText(_translate("MediaControlView", "00:00 / 00:00"))
        self.sliderVolumeLocal.setToolTip(_translate("MediaControlView", "Preview volume level"))
        self.sliderVolumeRemote.setToolTip(_translate("MediaControlView", "Remote volume level"))
        self.buttonMuteLocal.setToolTip(_translate("MediaControlView", "Preview mute"))
        self.labelVolumeRemote.setText(_translate("MediaControlView", "Remote:"))
        self.labelVolumeLocal.setText(_translate("MediaControlView", "Preview:"))
        self.buttonMuteRemote.setToolTip(_translate("MediaControlView", "Remote mute"))
        self.buttonStop.setToolTip(_translate("MediaControlView", "Stop"))
        self.sliderSeek.setToolTip(_translate("MediaControlView", "Video position"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright by: P.J. Grochowski

import logging


class Loggable:

    @property
    def log(self) -> logging.Logger:
        return logging.getLogger(f"{self.__class__.__name__}:({hex(id(self))})")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright by: P.J. Grochowski

import shutil
import sys
from pathlib import Path
from typing import Type

from setuptools import Command, find_packages
from setuptools import setup
from setuptools.command.build_py import build_py

NAME_PACKAGE_ROOT = 'pkg'

DIR_ROOT = Path(__file__).resolve().parent
DIR_PACKAGE_ROOT = DIR_ROOT / NAME_PACKAGE_ROOT

REL_PATH_DATA_FILES = Path('build') / 'data_files'

sys.path.append(str(DIR_PACKAGE_ROOT))

from kast import __app_description__, __app_name__, __email__, __version__, __author__, __package_name__, __app_url__


def assert_working_dir():
    currDir = Path().resolve()
    if currDir != DIR_ROOT:
        print(f"[ERROR]: Please change working directory: '{currDir}' -> '{DIR_ROOT}'")
        sys.exit(1)


def get_requirements():
    fileRequirements = DIR_ROOT / 'requirements.txt'

    if not fileRequirements.exists():
        print(f"<!> Could not find '{fileRequirements.name}' file. It will be generated.")
        from generate_requirements import generate
        generate()

    with open(fileRequirements) as fInput:
        return fInput.read().splitlines()


def get_long_description():
    fileReadme = DIR_ROOT / 'README.md'

    with open(fileReadme) as fInput:
        return fInput.read()


def prepare_data_files():
    dirDataFiles = DIR_ROOT / REL_PATH_DATA_FILES

    dirDataFiles.mkdir(parents=True, exist_ok=True)

    fileIconSrc = DIR_PACKAGE_ROOT / __package_name__ / 'assets' / 'appicon.png'
    fileIconDst = dirDataFiles / f'{__package_name__}.png'
    shutil.copyfile(fileIconSrc, fileIconDst)

    fileDesktop = dirDataFiles / f'{__package_name__}.desktop'
    with open(fileDesktop, 'w') as fOutput:
        fOutput.write(
            "[Desktop Entry]\n"
            "Type=Application\n"
            "Encoding=UTF-8\n"
            f"Name={__app_name__}\n"
            f"Icon={fileIconDst.name}\n"
            f"Exec={__package_name__}\n"
            "Categories=AudioVideo;\n"
            "Terminal=false\n"
        )


assert_working_dir()

kwargs = {'cmdclass': {}}


def add_cmd(cmd: str, cmdType: Type) -> None:
    kwargs['cmdclass'][cmd] = cmdType


try:
    from setup_qt import build_qt

    add_cmd('build_qt', build_qt)
    kwargs['options'] = {
        'build_qt': {
            'packages': [f'{NAME_PACKAGE_ROOT}/{__package_name__}'],
            'filename_ui': '{name}.py',
        },
    }

except ImportError:
    print("<!> Could not import 'setup_qt'. Some development feature will not be available!")


class CmdDistclean(Command):
    description = "removes 'build' and 'dist' dirs regardless of their contents"
    user_options = []

    def initialize_options(self) -> None:
        pass

    def finalize_options(self) -> None:
        pass

    def run(self) -> None:
        self.removeIfExists(DIR_ROOT / 'build')
        self.removeIfExists(DIR_ROOT / 'dist')

    def removeIfExists(self, path: Path) -> None:
        if path.exists():
            print(f"Removing: {path}")
            shutil.rmtree(path)


class CmdBuild(build_py):
    def run(self):
        prepare_data_files()
        super().run()


add_cmd('distclean', CmdDistclean)
add_cmd('build_py', CmdBuild)

setup(
    name=__package_name__,
    version=__version__,
    license="MIT",
    url=__app_url__,
    author=__author__,
    author_email=__email__,
    description=__app_description__,
    long_description=get_long_description(),
    long_description_content_type='text/markdown',
    package_dir={'': NAME_PACKAGE_ROOT},
    packages=find_packages(NAME_PACKAGE_ROOT),
    include_package_data=True,
    zip_safe=False,
    install_requires=get_requirements(),
    scripts=[f'script/{__package_name__}'],
    data_files=[
        ('share/applications', [str(REL_PATH_DATA_FILES / f'{__package_name__}.desktop')]),
        ('share/pixmaps', [str(REL_PATH_DATA_FILES / f'{__package_name__}.png')]),
    ],
    python_requires='>=3.7',
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    **kwargs
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright by: P.J. Grochowski

from dataclasses import dataclass, field
from typing import List


@dataclass
class MetaData:
    title: str = ''
    audioStreamLangs: List[str] = field(default_factory=lambda: [])
    subtitleStreamLangs: List[str] = field(default_factory=lambda: [])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright by: P.J. Grochowski

from enum import Enum
from typing import Callable, Optional

from PyQt5.QtCore import QUrl
from PyQt5.QtGui import QPixmap
from PyQt5.QtMultimedia import QMediaContent, QMediaPlayer
from PyQt5.QtMultimediaWidgets import QVideoWidget
from PyQt5.QtWidgets import QWidget

from kast.interface.qt5.UiServices import UiServices
from kast.interface.qt5.view.VideoPreviewView import Ui_VideoPreviewView
from kast.interface.qt5.viewModel.ViewModelBase import ViewBase, ViewModelBase
from kast.media.casting.CastState import CastPlayerState, Seconds
from kast.utils.Loggable import Loggable
from kast.utils.chrono import Milliseconds

MuteCallback = Callable[[bool], None]
VolumeCallback = Callable[[int], None]


class View(ViewBase, QWidget, Ui_VideoPreviewView):
    pass


class VideoPreviewViewModel(Loggable, ViewModelBase):

    POSITION_DELTA = 1

    def __init__(self, parent: QWidget, uiServices: UiServices) -> None:
        self._view = View(parent=parent)
        super().__init__(uiServices=uiServices, view=self._view)

        self._lastPlayerState = CastPlayerState.Unknown
        self._muteCallback: Optional[MuteCallback] = None
        self._volumeCallback: Optional[VolumeCallback] = None

        self._view.labelImageLogo.setPixmap(QPixmap(str(self.services.settings.appIconPath)))

        self._videoWidget = QVideoWidget(self._view)
        self._videoWidget.setVisible(False)

        self._view.layoutPreview.addWidget(self._videoWidget)

        self._mediaPlayer: Optional[QMediaPlayer] = None

        self._resetMediaPlayer()

    def _resetMediaPlayer(self, init: bool = True) -> None:
        if self._mediaPlayer is not None:
            self._mediaPlayer.setMedia(QMediaContent())
            self._mediaPlayer.setVideoOutput(None)
            self._mediaPlayer.deleteLater()

        if not init:
            return

        self._mediaPlayer = QMediaPlayer(self._view, QMediaPlayer.VideoSurface)
        self._mediaPlayer.setVideoOutput(self._videoWidget)

        self._mediaPlayer.error.connect(self._signalOnMediaPlayerError)
        self._mediaPlayer.mutedChanged.connect(self._signalOnMuted)
        self._mediaPlayer.volumeChanged.connect(self._signalOnVolume)

        self._mediaPlayer.setMuted(True)

    @property
    def muted(self) -> bool:
        return self._mediaPlayer.isMuted()

    @muted.setter
    def muted(self, value: bool) -> None:
        self._mediaPlayer.setMuted(value)

    @property
    def volume(self) -> int:
        return self._mediaPlayer.volume()

    @volume.setter
    def volume(self, value: int) -> None:
        self._mediaPlayer.setVolume(value)

    def setMuteCallback(self, callback: MuteCallback) -> None:
        self._muteCallback = callback

    def setVolumeCallback(self, callback: VolumeCallback) -> None:
        self._volumeCallback = callback

    def cleanup(self) -> None:
        self._resetMediaPlayer(init=False)

    def enable(self, state: bool) -> None:
        self._view.labelImageLogo.setVisible(not state)
        self._videoWidget.setVisible(state)

        self._reloadMedia(enabled=state)

    def setPlayerState(self, state: CastPlayerState) -> None:
        if not self._hasMedia() or state == self._lastPlayerState:
            return
        self._lastPlayerState = state

        if state == CastPlayerState.Playing:
            self._mediaPlayer.play()
            return

        if state in [CastPlayerState.Buffering, CastPlayerState.Paused]:
            self._mediaPlayer.pause()
            return

        self._mediaPlayer.stop()

    def updateVideoPosition(self, position: Seconds) -> None:
        if not self._hasMedia():
            return

        previewPosition = Seconds(Milliseconds(self._mediaPlayer.position()))
        if int(previewPosition) not in self._designateAcceptablePositionRange(position):
            self.log.info(f"Updating preview position: {previewPosition} -> {position}")
            self._mediaPlayer.setPosition(Milliseconds(position).value)

    def _designateAcceptablePositionRange(self, position: Seconds) -> range:
        return range(
            int(position) - self.POSITION_DELTA,
            int(position) + self.POSITION_DELTA + 1
        )

    def _reloadMedia(self, enabled: bool) -> None:
        if(
            (enabled and not self._hasMedia()) or
            (not enabled and self._hasMedia())
        ):
            self.log.info("Reloading a media content.")
            self._resetMediaPlayer()

            if enabled:
                mediaContent = QMediaContent(QUrl.fromLocalFile(str(self.services.mediaServer.mediaContent.movieFile)))
                self._mediaPlayer.setMedia(mediaContent)

    def _hasMedia(self) -> bool:
        return not self._mediaPlayer.media().isNull()

    def _signalOnMediaPlayerError(self) -> None:
        self.log.error(f"Media player error: {mediaPlayerErrorToStr(self._mediaPlayer.error())}")

    def _signalOnMuted(self, muted: bool) -> None:
        self._muteCallback and self._muteCallback(muted)

    def _signalOnVolume(self, volume: int) -> None:
        self._volumeCallback and self._volumeCallback(volume)


def mediaPlayerErrorToStr(errorCode: int) -> str:
    class Error(Enum):
        NoError = QMediaPlayer.NoError
        ResourceError = QMediaPlayer.ResourceError
        FormatError = QMediaPlayer.FormatError
        NetworkError = QMediaPlayer.NetworkError
        AccessDeniedError = QMediaPlayer.AccessDeniedError
        ServiceMissingError = QMediaPlayer.ServiceMissingError

    if errorCode not in set(er.value for er in Error):
        return f'Unknown({errorCode})'

    return Error(errorCode).name



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pkg/kast/interface/qt5/view/AboutViewModel.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutViewModel(object):
    def setupUi(self, AboutViewModel):
        AboutViewModel.setObjectName("AboutViewModel")
        AboutViewModel.setWindowModality(QtCore.Qt.NonModal)
        AboutViewModel.resize(136, 170)
        AboutViewModel.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        AboutViewModel.setSizeGripEnabled(False)
        AboutViewModel.setModal(True)
        self._layout = QtWidgets.QVBoxLayout(AboutViewModel)
        self._layout.setContentsMargins(6, 6, 6, 6)
        self._layout.setObjectName("_layout")
        self._layoutContent = QtWidgets.QHBoxLayout()
        self._layoutContent.setObjectName("_layoutContent")
        self.labelLogo = QtWidgets.QLabel(AboutViewModel)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelLogo.sizePolicy().hasHeightForWidth())
        self.labelLogo.setSizePolicy(sizePolicy)
        self.labelLogo.setText("")
        self.labelLogo.setObjectName("labelLogo")
        self._layoutContent.addWidget(self.labelLogo)
        self._layoutLabels = QtWidgets.QVBoxLayout()
        self._layoutLabels.setObjectName("_layoutLabels")
        self._labelAppName = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAppName.setFont(font)
        self._labelAppName.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAppName.setObjectName("_labelAppName")
        self._layoutLabels.addWidget(self._labelAppName)
        self._labelAppVersion = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAppVersion.setFont(font)
        self._labelAppVersion.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAppVersion.setObjectName("_labelAppVersion")
        self._layoutLabels.addWidget(self._labelAppVersion)
        self._labelAppDescription = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAppDescription.setFont(font)
        self._labelAppDescription.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAppDescription.setObjectName("_labelAppDescription")
        self._layoutLabels.addWidget(self._labelAppDescription)
        self._labelAuthorName = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAuthorName.setFont(font)
        self._labelAuthorName.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAuthorName.setObjectName("_labelAuthorName")
        self._layoutLabels.addWidget(self._labelAuthorName)
        self._labelAuthorContact = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAuthorContact.setFont(font)
        self._labelAuthorContact.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAuthorContact.setObjectName("_labelAuthorContact")
        self._layoutLabels.addWidget(self._labelAuthorContact)
        self._layoutContent.addLayout(self._layoutLabels)
        self._spacer = QtWidgets.QFrame(AboutViewModel)
        self._spacer.setFrameShape(QtWidgets.QFrame.VLine)
        self._spacer.setFrameShadow(QtWidgets.QFrame.Sunken)
        self._spacer.setObjectName("_spacer")
        self._layoutContent.addWidget(self._spacer)
        self._layoutDetails = QtWidgets.QVBoxLayout()
        self._layoutDetails.setObjectName("_layoutDetails")
        self.labelAppName = QtWidgets.QLabel(AboutViewModel)
        self.labelAppName.setObjectName("labelAppName")
        self._layoutDetails.addWidget(self.labelAppName)
        self.labelAppVersion = QtWidgets.QLabel(AboutViewModel)
        self.labelAppVersion.setObjectName("labelAppVersion")
        self._layoutDetails.addWidget(self.labelAppVersion)
        self.labelAppDescription = QtWidgets.QLabel(AboutViewModel)
        self.labelAppDescription.setObjectName("labelAppDescription")
        self._layoutDetails.addWidget(self.labelAppDescription)
        self.labelAuthorName = QtWidgets.QLabel(AboutViewModel)
        self.labelAuthorName.setObjectName("labelAuthorName")
        self._layoutDetails.addWidget(self.labelAuthorName)
        self.labelAuthorContact = QtWidgets.QLabel(AboutViewModel)
        self.labelAuthorContact.setObjectName("labelAuthorContact")
        self._layoutDetails.addWidget(self.labelAuthorContact)
        self._layoutContent.addLayout(self._layoutDetails)
        self._layoutContent.setStretch(3, 1)
        self._layout.addLayout(self._layoutContent)
        self._buttonBox = QtWidgets.QDialogButtonBox(AboutViewModel)
        self._buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self._buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self._buttonBox.setObjectName("_buttonBox")
        self._layout.addWidget(self._buttonBox)

        self.retranslateUi(AboutViewModel)
        self._buttonBox.accepted.connect(AboutViewModel.accept)
        self._buttonBox.rejected.connect(AboutViewModel.reject)
        QtCore.QMetaObject.connectSlotsByName(AboutViewModel)

    def retranslateUi(self, AboutViewModel):
        _translate = QtCore.QCoreApplication.translate
        AboutViewModel.setWindowTitle(_translate("AboutViewModel", "About"))
        self._labelAppName.setText(_translate("AboutViewModel", "Application:"))
        self._labelAppVersion.setText(_translate("AboutViewModel", "Version:"))
        self._labelAppDescription.setText(_translate("AboutViewModel", "Description:"))
        self._labelAuthorName.setText(_translate("AboutViewModel", "Author:"))
        self._labelAuthorContact.setText(_translate("AboutViewModel", "Contact:"))
        self.labelAppName.setText(_translate("AboutViewModel", "..."))
        self.labelAppVersion.setText(_translate("AboutViewModel", "..."))
        self.labelAppDescription.setText(_translate("AboutViewModel", "..."))
        self.labelAuthorName.setText(_translate("AboutViewModel", "..."))
        self.labelAuthorContact.setText(_translate("AboutViewModel", "..."))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright by: P.J. Grochowski

import os
from typing import Type

from kast.interface.overlay.MediaOverlayBase import MediaOverlayBase


def _getImpl() -> Type[MediaOverlayBase]:
    if os.name == 'posix':
        from ._mpris.MediaOverlayMpris import MediaOverlayMpris
        return MediaOverlayMpris

    if os.name == 'nt':
        from ._winrt.MediaOverlayWinrt import MediaOverlayWinrt
        return MediaOverlayWinrt

    return MediaOverlayBase


MediaOverlay = _getImpl()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright by: P.J. Grochowski

from mpris_server.events import EventAdapter
from mpris_server.server import Server as MprisServer

from kast.interface.overlay._mpris.MprisCastController import MprisCastController
from kast.media.casting.CastState import CastState
from kast.settings.SettingsState import SettingsState


class MprisEventListener(EventAdapter):

    def __init__(
        self,
        mprisServer: MprisServer,
        mprisCastController: MprisCastController
    ) -> None:
        super().__init__(
            player=mprisServer.player,
            root=mprisServer.root
        )
        self._mprisCastController = mprisCastController

    def onSettingsEvent(self, event: SettingsState) -> None:
        self._mprisCastController.onSettingsEvent(event=event)
        self._reload()

    def onCastEvent(self, event: CastState) -> None:
        self._reload()

    def _reload(self) -> None:
        self.on_volume()
        self.on_playback()
        self.on_title()
        self.on_options()
        self.on_playpause()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright by: P.J. Grochowski

from __future__ import annotations  # TODO: Remove when support dropped for: Python < 3.10

from dataclasses import dataclass
from functools import total_ordering
from math import floor
from typing import Optional, Union


class TimeUnitTypeError(TypeError):
    pass


@total_ordering
class TimeUnit:

    ConvFactor = float
    NumericValue = int

    DEFAULT_VALUE = 0

    @staticmethod
    def Value() -> Union:
        return Union[int, float, TimeUnit]

    @dataclass(frozen=True)
    class Params:
        name: str
        convFactor: TimeUnit.ConvFactor

    class Type:

        def __init__(self, name: str, convFactor: TimeUnit.ConvFactor) -> None:
            self._timeUnit = TimeUnit(params=TimeUnit.Params(name=name, convFactor=convFactor))

        def __call__(self, value: Optional[TimeUnit.Value()] = None) -> TimeUnit:
            return self._timeUnit(value=value)

        @property
        def convFactor(self):
            return self._timeUnit.convFactor

    def __init__(
        self,
        params: Params,
        value: Optional[TimeUnit.Value()] = None
    ) -> None:
        value = value if value is not None else self.DEFAULT_VALUE

        self._params = params
        self._value = self._convertValue(value)

    def __call__(self, value: Optional[TimeUnit.Value()] = None) -> TimeUnit:
        return TimeUnit(params=self._params, value=value)

    def __int__(self) -> int:
        return self.value

    def __str__(self) -> str:
        return f"{self.name}({self.value})"

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}({self.name}={self.value}) at {hex(id(self))}>"

    def __add__(self, other: Value()) -> TimeUnit:
        if isinstance(other, int):
            return TimeUnit(params=self._params, value=(self.value + other))

        if isinstance(other, TimeUnit):
            return TimeUnit(params=self._params, value=(self.value + self._convertValue(other)))

        raise TimeUnitTypeError(f"Cannot increase '{self.name}' with '{type(other).__name__}'!")

    def __sub__(self, other: Value()) -> TimeUnit:
        return self.__add__(-other)

    def __neg__(self) -> TimeUnit:
        return TimeUnit(params=self._params, value=(-self.value))

    def __eq__(self, other: Value()) -> bool:
        return self.value == self._convertValue(other)

    def __gt__(self, other: Value()) -> bool:
        return self.value > self._convertValue(other)

    @property
    def name(self) -> str:
        return self._params.name

    @property
    def convFactor(self) -> ConvFactor:
        return self._params.convFactor

    @property
    def value(self) -> NumericValue:
        return self._value

    def _convertValue(self, value: Value()) -> NumericValue:
        if not isinstance(value, self.Value().__args__):
            raise TimeUnitTypeError(f"Cannot create '{self._params.name}' from '{type(value).__name__}'!")

        if isinstance(value, TimeUnit):
            return int(floor(value.value * value.convFactor / self.convFactor))

        return int(floor(value))


Seconds = TimeUnit.Type(name='Seconds', convFactor=1)

Milliseconds = TimeUnit.Type(name='Milliseconds', convFactor=(Seconds.convFactor / 1_000))
Microseconds = TimeUnit.Type(name='Microseconds', convFactor=(Milliseconds.convFactor / 1_000))
Nanoseconds = TimeUnit.Type(name='Nanoseconds', convFactor=(Microseconds.convFactor / 1_000))

Minutes = TimeUnit.Type(name='Minutes', convFactor=(60 * Seconds.convFactor))
Hours = TimeUnit.Type(name='Hours', convFactor=(60 * Minutes.convFactor))
Days = TimeUnit.Type(name='Days', convFactor=(24 * Hours.convFactor))
Weeks = TimeUnit.Type(name='Weeks', convFactor=(7 * Days.convFactor))
